<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Teacher_Discipline_Edit extends Controller_Environment_Teacher
{
    /** @var  Model_Discipline */
    private $discipline;

    const twigSource = 'teacher/discipline/edit/';


    public function before() {
        parent::before();

        $id = (int) $this->request->param('id');
        $this->discipline = Model_Discipline::load($id);
        if ($this->discipline->AuthorID != $this->user->TeacherID)
            throw HTTP_Exception::factory(403, "Недостаточно прав для редактирования.");

        $section = $this->request->param('section');
        $this->twig->set_filename(static::twigSource . $section);
        $this->twig->Discipline = $this->discipline;
    }


    public function action_edit_settings() {
        $this->twig->SubjectsList = $this->user->Faculty->getSubjects();
        $this->twig->GradesList   = Model_Grades::loadAll();
    }

    public function action_edit_structure() {
        $map = Model_Map::getMapForDiscipline($this->discipline->ID);
        $this->twig->Map = $this->getMapInfo($map);
    }
    
    public function action_edit_groups() {
        $this->twig->set([
            'GroupsForDiscipline' => $this->discipline->getGroups(),
            'Groups'              => $this->user->Faculty->getGroups($this->discipline->GradeID),
        ]);
    }
    
    public function action_edit_students() {
        $this->manageGroups($this->discipline, $students, $attached);

        $this->twig->set([
            'Groups'         => $students,
            'GroupsAttached' => $attached,
            'GradesList'     => Model_Grades::loadAll(),
            'GroupsList'     => Model_Faculty::with($this->discipline->FacultyID)
                                    ->getGroups($this->discipline->GradeID),
        ]);
    }
    
    public function action_edit_teachers() {
        $this->twig->set([
            'FacultiesList'    => Model_Faculties::load(),
            'Departments'      => $this->user->Faculty->getDepartments(),
            'BindTeachersList' => Model_Teachers::getNamesForDiscipline($this->discipline->ID),
        ]);
    }


    private function getMapInfo($map) {
        $maxRate = $i = $module = 0;
        $mapHandled = [];

        foreach ($map as $row) {
            if ($row['ModuleType'] == 'extra') {
                continue;
            }

            if ($row['ModuleID'] != $module) {
                $i++;
                $module = $row['ModuleID'];
                $mapHandled[$i]['ModuleID'] = $row['ModuleID'];
                $mapHandled[$i]['ModuleType'] = $row['ModuleType'];
                if ($row['ModuleType'] == 'bonus') {
                    $mapHandled['isSetBonus'] = true;
                }
                $mapHandled[$i]['CurrentControl'] = 0;
                $mapHandled[$i]['LandmarkControl'] = 0;
            }

            if (!isset($mapHandled[$i]['SubmodulesCount'])) {
                $mapHandled[$i]['SubmodulesCount'] = 0;
                $mapHandled[$i]['MaxRate'] = 0;
            }

            $mapHandled[$i]['IsExam'] = $row['IsExam'];
            $j = $mapHandled[$i]['SubmodulesCount'] += 1;
            $mapHandled[$i]['MaxRate'] += (int)$row['MaxRate'];
            $mapHandled[$i]['ModuleTitle'] = $row['ModuleName'];
            $mapHandled[$i][$j]['SubmoduleID'] = $row['SubmoduleID'];
            $mapHandled[$i][$j]['Title'] = $row['SubmoduleName'];
            $mapHandled[$i][$j]['Description'] = $row['SubmoduleDescription'];
            $mapHandled[$i][$j]['SubmoduleControl'] = $row['SubmoduleType'];
            $mapHandled[$i][$j]['MaxRate'] = (int)$row['MaxRate'];
            $maxRate += $row['MaxRate'];

            if ($row['SubmoduleType'] == 'CurrentControl') {
                $mapHandled[$i]['CurrentControl'] += (int)$row['MaxRate'];
            } elseif ($row['SubmoduleType'] == 'LandmarkControl') {
                $mapHandled[$i]['LandmarkControl'] += (int)$row['MaxRate'];
            }
        }

        $mapHandled['ModulesCount'] = $i;
        $mapHandled['MaxRate'] = (int)$maxRate;
        if ($mapHandled['isSetBonus'] == true) {
            $mapHandled['MaxRate'] = $mapHandled['MaxRate'] - 10;
        }

        return $mapHandled;
    }

    private function manageGroups(Model_Discipline $discipline, &$std, &$attached) {
        $students = Model_Students::ofDiscipline($discipline->ID);
        $groupID = $i = $k = $n = $j = 0;
        $attached = $std = [];

        foreach ($students as $row) {
            if ($row['GroupID'] != $groupID) {
                $groupID = $row['GroupID'];

                if ($row['AttachType'] == 'attach') {
                    $attached[++$k] = $row;
                } else {
                    $std[++$i] = $row;
                }
            }

            if ($row['AttachType'] == 'attach') {
                $attached[$k]['students'][] = $row;
            } else {
                $std[$i]['students'][] = $row;
            }
        }
    }
}
            